{
  The simple demo of sound processing and visualization
  Author:  Eugen Tarasov

  This demo is based on the DelphiX sample and library.
  There is only 16 bit wave support

  Last modified - Jule, 2004

  Please send all suggestions to author

  e-mail: dsplab@teworks.com
  http://www.teworks.com
}

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, ComCtrls, DXSounds, MMSystem, Wave, AudioStream,
  Math, dspIIRfilters, Buttons, syncobjs, dspFFT, dspTypes, dspWindow, SpecList;

type
  TBars = class(TGraphicControl)
  private
    Bmp: TBitmap;
    BrushBitmap: TBitmap;
  public
    Values: TSpectrumArray;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure Reset;
  end;


  TMainForm = class(TForm)
    Timer: TTimer;
    OpenDialog: TOpenDialog;
    StatusBar1: TStatusBar;
    Panel2: TPanel;
    Panel1: TPanel;
    PanTrackBar: TTrackBar;
    PanLabel: TLabel;
    VolumeLabel: TLabel;
    ProgressLabel: TLabel;
    BytesLabel: TLabel;
    PlayButton: TButton;
    PauseButton: TButton;
    StopButton: TButton;
    Panel3: TPanel;
    VolumeTrackBar: TTrackBar;
    Panel4: TPanel;
    ProgressTrackBar: TTrackBar;    Label1: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Panel5: TPanel;
    DisplayPanel: TPanel;
    Bevel4: TBevel;
    Timer1: TTimer;
    OpenButton: TButton;
    SpeedButton1: TSpeedButton;
    Panel7: TPanel;
    Bevel5: TBevel;
    Panel8: TPanel;
    TBBass: TTrackBar;
    Label2: TLabel;
    Panel18: TPanel;
    TrackBar2: TTrackBar;
    Panel19: TPanel;
    TBTreble: TTrackBar;
    Label12: TLabel;
    LPLeft: TdspIIRFilter;
    LPRight: TdspIIRFilter;
    HPLeft: TdspIIRFilter;
    HPRight: TdspIIRFilter;
    Panel6: TPanel;
    Panel9: TPanel;
    Label3: TLabel;
    Bevel6: TBevel;
    TB1: TTrackBar;
    Panel10: TPanel;
    Label4: TLabel;
    Bevel7: TBevel;
    TB2: TTrackBar;
    Panel11: TPanel;
    Label5: TLabel;
    Bevel8: TBevel;
    TB3: TTrackBar;
    Panel12: TPanel;
    Label6: TLabel;
    Bevel9: TBevel;
    TB6: TTrackBar;
    Panel13: TPanel;
    Label7: TLabel;
    Bevel10: TBevel;
    TB7: TTrackBar;
    Panel14: TPanel;
    Label8: TLabel;
    Bevel11: TBevel;
    TB5: TTrackBar;
    Panel15: TPanel;
    Label9: TLabel;
    Bevel12: TBevel;
    TB4: TTrackBar;
    Panel16: TPanel;
    Label10: TLabel;
    Bevel13: TBevel;
    TB9: TTrackBar;
    Panel17: TPanel;
    Label11: TLabel;
    Bevel14: TBevel;
    TB8: TTrackBar;
    cbClip: TCheckBox;
    Panel20: TPanel;
    TBPreamp: TTrackBar;
    Label13: TLabel;
    FFTLeft: TdspFFT;
    FFTRight: TdspFFT;
    FFTEq: TdspFFT;
    FFTSpectrum1: TdspFFT;
    FFTSpectrum2: TdspFFT;
    PLeftDisp: TPanel;
    PRightDisp: TPanel;
    procedure TimerTimer(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure FileOpenClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure PanTrackBarChange(Sender: TObject);
    procedure VolumeTrackBarChange(Sender: TObject);
    procedure PauseButtonClick(Sender: TObject);
    procedure DXSoundFinalize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DisplayPanelResize(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure TimbreChange(Sender: TObject);
    procedure cbClipClick(Sender: TObject);

  private { Playing sound }
    FUpdating: Boolean;

    DXSound: TDXSound;
    Audio: TdspAudioFileStream;

    procedure CloseSound;
    procedure UpdateInfo;
    procedure UpdateFileInfo;

    procedure ProgressMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ProgressMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);

  private  { sound processing }
    CS: TCriticalSection;

    Clipped: Boolean;
    Preamp: Single;
    PeakGainLeft, PeakGainRight: Single;
    BassGain, TrebleGain: Single;

    SamplesPerSec: Integer;
    BitsPerSample: Integer;
    NumOfChannels: Integer;

    AmpCoeffs: TSpectrumArray;
    OverlapL, OverlapR: PdspSingleArray;
    FilterLen, FilterDataSize: Integer;

    procedure SetupProcessingParameters;
    { Called when the next peace of data is required }
    procedure OnDataRead(var Buffer; Count: Longint);

  private { the display }
    SpectrumValues: TSpectrumList;
    LeftDisp, RightDisp: TBars;
  end;

var
  MainForm: TMainForm;

implementation

uses About;

{$R *.DFM}

type
  TMyTrackBar = class(TTrackBar)
  public
    property OnMouseDown;
    property OnMouseUp;
  end;


constructor TBars.Create(AOwner: TComponent);
begin
  inherited;
  Bmp:= TBitmap.Create;

  BrushBitmap:= TBitmap.Create;
  BrushBitmap.PixelFormat:= pf24bit;
  BrushBitmap.Width:= 8; BrushBitmap.Height:= 8;
  with BrushBitmap.Canvas do
  begin
    Pen.Width:= 2;
    Pen.Color:= $009A6D30;
    Brush.Color:= clBtnFace;
    FillRect(Rect(0, 0, 8, 8));
    MoveTo(-1, 1); LineTo(9, 1);
    MoveTo(-1, 4); LineTo(9, 4);
  end;

  Reset;
end;


destructor TBars.Destroy;
begin
  Bmp.Free;
  BrushBitmap.Free;
  inherited;
end;


procedure TBars.Paint;
var
  I: Integer;
  R: TRect;
  XD: Single;

  function Translate(V: Single): Integer;
  begin
    Result:= -Round(V * Height / 40);
  end;
begin
  Bmp.Width:= Width;
  Bmp.Height:= Height;

  XD:= Width / cNumberOfSpectrumChannels;
  Bmp.Canvas.Brush.Color:= Parent.Brush.Color;
  Bmp.Canvas.FillRect(Rect(0, 0, Width, Height));

  Bmp.Canvas.Brush.Bitmap:= BrushBitmap;
  for I:= 0 to cNumberOfSpectrumChannels - 1 do
  begin
    R.Left:= Round(I*XD); R.Right:= Round(I*XD + XD) - 4;
    R.Bottom:= Height;
    R.Top:= Translate(20 * log10((Values[I] + 1) / High(SmallInt)));
    Bmp.Canvas.FillRect(R);
  end;

  Canvas.CopyRect(ClientRect, Bmp.Canvas, ClientRect);
end;


procedure TBars.Reset;
var I: Integer;
begin
  for I:= Low(Values) to High(Values) do Values[I]:= 0;
end;

{==============================================================================}
{ Player                                                                       }
{==============================================================================}

procedure TMainForm.FormCreate(Sender: TObject);
var I: Integer;
begin
  Application.Title:= 'DSPLab - ' + Caption;

  CS:= TCriticalSection.Create;

  {Create spectrum display}
  LeftDisp:= TBars.Create(Self);
  LeftDisp.Parent:= PLeftDisp;
  LeftDisp.Align:= alClient;

  RightDisp:= TBars.Create(Self);
  RightDisp.Parent:= PRightDisp;
  RightDisp.Align:= alClient;

  SpectrumValues:= TSpectrumList.Create;

  { Initialization for sound processing }

  Clipped:= true;
  Preamp:= 1;
  PeakGainLeft:= 1;
  PeakGainRight:= 1;

  for I:= Low(AmpCoeffs) to High(AmpCoeffs) do AmpCoeffs[I]:= 1;

  { Initialization for sound playing }
  DXSound:= TDXSound.Create(Self);
  DXSound.Options:= [soGlobalFocus];
  DXSound.OnFinalize:= DXSoundFinalize;

  DisplayPanelResize(Self);
  TimbreChange(Self);
  Timer1.Enabled:= true;
  UpdateInfo;
end;


procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Timer.Enabled:= false;
  Timer1.Enabled:= false;
  CloseSound;
  DXSound.Free;
  CS.Free;
  SpectrumValues.Free;
end;


procedure TMainForm.DXSoundFinalize(Sender: TObject);
begin
  Audio.Free;
  Audio:= nil;
end;


procedure TMainForm.FileOpenClick(Sender: TObject);
var WaveFormat: TWaveFormatEx;
begin
  if not OpenDialog.Execute then Exit;

  CloseSound;

  try
    Caption := Application.Title + ' - ' + ExtractFileName(OpenDialog.FileName);
    DXSound.Initialize;

    { Create the audio stream}
    Audio:= TdspAudioFileStream.Create(DXSound);
    Audio.BufferLength := 1000; // Buffer Length = 1 second
    Audio.FileName     := OpenDialog.FileName;
    Audio.OnDataProcessing:= OnDataRead;

    SamplesPerSec      := Audio.Format^.nSamplesPerSec;
    BitsPerSample      := Audio.Format^.wBitsPerSample;
    NumOfChannels      := Audio.Format^.nChannels;

    {  Setting of format of primary buffer.  }
    MakePCMWaveFormatEx(WaveFormat, 44100, 16, 2);
    DXSound.Primary.SetFormat(WaveFormat);
    SetupProcessingParameters;
    UpdateFileInfo;

    PlayButton.Enabled       := True;
    PauseButton.Enabled      := False;
    StopButton.Enabled       := False;
    PanTrackBar.Enabled      := True;
    VolumeTrackBar.Enabled   := True;
    ProgressTrackBar.Enabled := True;
    ProgressLabel.Enabled    := True;
    BytesLabel.Enabled       := True;

    FUpdating:= true;
    try
      PanTrackBar.Position      := Audio.Pan;
      VolumeTrackBar.Position   := Audio.Volume;
      ProgressTrackBar.Max      := Audio.Size;
      ProgressTrackBar.PageSize := ProgressTrackBar.Max div 25;
      TMyTrackBar(ProgressTrackBar).OnMouseUp:= ProgressMouseUp;
      TMyTrackBar(ProgressTrackBar).OnMouseDown:= ProgressMouseDown;
    finally
      FUpdating:= false;
    end;
  except
    CloseSound;
    raise;
  end;
end;


procedure TMainForm.CloseSound;
begin
  StopButtonClick(nil);
  Timer.Enabled := False;

  DXSound.Finalize;
  Caption:= Application.Title;

  PlayButton.Enabled       := False;
  PauseButton.Enabled      := False;
  StopButton.Enabled       := False;
  PanTrackBar.Enabled      := False;
  VolumeTrackBar.Enabled   := False;
  ProgressTrackBar.Enabled := False;
  ProgressLabel.Enabled    := false;
  BytesLabel.Enabled       := false;

  UpdateInfo;
  UpdateFileInfo;

  FreeMem(OverlapL);
  FreeMem(OverlapR);
end;


procedure TMainForm.PanTrackBarChange(Sender: TObject);
begin
  if FUpdating then Exit;
  Audio.Pan := PanTrackBar.Position * 100;
  UpdateInfo;
end;


procedure TMainForm.VolumeTrackBarChange(Sender: TObject);
begin
  if FUpdating then Exit;
  Audio.Volume := VolumeTrackBar.Position * 100;
  UpdateInfo;
end;


procedure TMainForm.ProgressMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Timer.Enabled:= false;
end;


procedure TMainForm.ProgressMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  CS.Acquire;
  try
    SpectrumValues.Clear;
  finally
    CS.Release;
  end;
  Timer.Enabled:= true;
  Audio.Position := ProgressTrackBar.Position;
  UpdateInfo;
end;


procedure TMainForm.PlayButtonClick(Sender: TObject);
begin
  Audio.Play;

  PlayButton.Enabled  := False;
  PauseButton.Enabled := True;
  StopButton.Enabled  := True;
  Timer.Enabled       := True;
end;


procedure TMainForm.PauseButtonClick(Sender: TObject);
begin
  Audio.Stop;

  PlayButton.Enabled  := True;
  PauseButton.Enabled := False;
  StopButton.Enabled  := False;
  Timer.Enabled       := False;
  UpdateInfo;
  SpectrumValues.Clear;
end;


procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  if Audio <> nil then
  begin
    Audio.Stop;
    Audio.Position := 0;
  end;

  PlayButton.Enabled  := True;
  PauseButton.Enabled := False;
  StopButton.Enabled  := False;
  Timer.Enabled       := False;
  UpdateInfo;

  SpectrumValues.Clear;
end;


procedure TMainForm.TimerTimer(Sender: TObject);
begin
  if Audio.Playing then UpdateInfo else
    if Audio.Position >= Audio.Size then StopButtonClick(Self);

end;


procedure TMainForm.DisplayPanelResize(Sender: TObject);
begin
  PLeftDisp.Width:= DisplayPanel.Width div 2;
end;


procedure TMainForm.UpdateInfo;
var P: Integer;
begin
  if (Audio <> nil) and (Audio.Format <> nil) then
  begin
    P:= (Audio.Position div Audio.Format.nBlockAlign) div SamplesPerSec;

    PanLabel.Caption      := Format('Pan: %ddB', [Audio.Pan div 100]);
    VolumeLabel.Caption   := Format('Volume: %ddB', [Audio.Volume div 100]);
    ProgressLabel.Caption := Format('Time: %.2d:%.2d:%.2d', [(P div 60) div 60, (P div 60) mod 60, P mod 60]);
    BytesLabel.Caption    := Format('%d Bytes', [Audio.Position]);

    FUpdating:= true;
    try
      ProgressTrackBar.Position := Audio.Position;
    finally
      FUpdating:= false;
    end;
  end else
  begin
    PanLabel.Caption      := 'Pan';
    VolumeLabel.Caption   := 'Volume';
    ProgressLabel.Caption := 'Time';
    BytesLabel.Caption    := 'Bytes';
  end;
end;


procedure TMainForm.UpdateFileInfo;
var FileInfo: String;
begin
  if Audio <> nil then
  begin
    FileInfo:= Format('%d kHz, %d bit, ', [SamplesPerSec,  BitsPerSample]);
    if NumOfChannels = 1 then FileInfo:= FileInfo + 'Mono'
    else FileInfo:= FileInfo + 'Stereo';

    StatusBar1.Panels[0].Width:= StatusBar1.Canvas.TextWidth(FileInfo) + 10;
    StatusBar1.Panels[0].Text:= FileInfo;
  end else StatusBar1.Panels[0].Width:= 0;
end;

{==============================================================================}
{ Sound Processing                                                             }
{==============================================================================}

procedure TMainForm.OnDataRead(var Buffer; Count: Longint);

  procedure CalculateSpectrum(FFT: TdspFFT);
  var
    S: Single;
    I, N1, N2: Integer;
    SpectrumItem: PSpectrumItem;
    SpectrumData: PdspSingle;
  begin
    FFT.FFT;
    FFT.CalculateMagnitudes;

    CS.Acquire;
    try
      SpectrumItem:= SpectrumValues.Add;
      SpectrumItem.Channel:= FFT.Tag;
      SpectrumItem.Position:= Audio.WaveStream.Position;
      SpectrumData:= PdspSingle(@SpectrumItem.Values[0]);
      N1:= 0; N2:= 1;
      while N1 < FFT.BufferSize2 do
      begin
        S:= 0;
        for I:= N1 to N2 - 1 do S:= S + FFT.RealOut[I];

        SpectrumData^:= S;
        Inc(SpectrumData);
        N1:= N2;
        N2:= N2 * 2;
      end;
    finally
      CS.Release;
    end;
  end;

  procedure FFTFilter(FFT: TdspFFT; OLAP: PdspSingleArray);
  var
    S: Single;
    I: Integer;
  begin
    FFT.FFT;

    Move(PdspSingle(FFT.RealOut)^, PdspSingle(FFT.RealIn)^, FFT.BufferSize2*Sizeof(Single));
    Move(PdspSingle(FFT.ImagOut)^, PdspSingle(FFT.ImagIn)^, FFT.BufferSize2*Sizeof(Single));

    for I:= 0 to FFT.BufferSize2 do
    begin
      S:=             FFT.RealIn[I] * FFTEq.RealOut[I] - FFT.ImagIn[I] * FFTEq.ImagOut[I];
      FFT.ImagIn[I]:= FFT.RealIn[I] * FFTEq.ImagOut[I] + FFT.ImagIn[I] * FFTEq.RealOut[I];
      FFT.RealIn[I]:= S;
    end;

    for I:= 1 to FFT.BufferSize2 do
    begin
      FFT.RealIn[FFT.BufferSize - I]:=  FFT.RealIn[I];
      FFT.ImagIn[FFT.BufferSize - I]:= -FFT.ImagIn[I];
    end;

    FFT.IFFT;

    for I:= 0 to FilterLen - 2 do FFT.RealOut[I]:= FFT.RealOut[I] + OLAP[I];
    for I:= 0 to FilterLen - 2 do OLAP[I]:= FFT.RealOut[FilterDataSize + I];
  end;


  { Process Mono }
  procedure ProcessMono;
  var
    I: Integer;
    VL, VGL: Single;
    PV: PSmallInt;
  begin
    PV:= PSmallInt(@Buffer);
    for I:= 0 to FilterDataSize - 1 do
    begin
      FFTLeft.RealIn[I]:= PV^; Inc(PV);
    end;

    FFTFilter(FFTLeft, OverlapL);

    PV:= PSmallInt(@Buffer);
    for I:= 0 to FilterDataSize - 1 do
    begin
      PeakGainLeft:= Min(1, PeakGainLeft + 0.000005);
      PeakGainRight:= Min(1, PeakGainRight + 0.000005);
      VL:= FFTLeft.RealOut[I];
      VL:= VL + LPLeft.Filter(VL) * BassGain + HPLeft.Filter(VL) * TrebleGain;
      VL:= VL * Preamp;

      if Clipped then
      begin
        if Abs(VL) > High(SmallInt) then
        begin
          VGL:= High(SmallInt) / Abs(VL);
          PeakGainLeft:= Min(PeakGainLeft, VGL);
        end;
        VL:= VL * PeakGainLeft;
      end else
      begin
        VL:=  Min(High(SmallInt), Max(VL, Low(SmallInt)));
      end;

      PV^:= Round(VL); Inc(PV);
    end;

    PV:= PSmallInt(@Buffer);
    for I:= 0 to Min(FilterDataSize, FFTSpectrum1.BufferSize) - 1 do
    begin
      FFTSpectrum1.RealIn[I]:= PV^; Inc(PV);
    end;

    CalculateSpectrum(FFTSpectrum1);
  end;

  { Process Stereo }
  procedure ProcessStereo;
  var
    I: Integer;
    VL, VR, VGL, VGR: Single;
    PV: PSmallInt;
  begin
    PV:= PSmallInt(@Buffer);
    for I:= 0 to FilterDataSize - 1 do
    begin
      FFTLeft.RealIn[I]:= PV^; Inc(PV);
      FFTRight.RealIn[I]:= PV^; Inc(PV);
    end;

    FFTFilter(FFTLeft, OverlapL);
    FFTFilter(FFTRight, OverlapR);

    PV:= PSmallInt(@Buffer);
    for I:= 0 to FilterDataSize - 1 do
    begin
      PeakGainLeft:= Min(1, PeakGainLeft + 0.000005);
      PeakGainRight:= Min(1, PeakGainRight + 0.000005);
      VL:= FFTLeft.RealOut[I];
      VL:= VL + lPLeft.Filter(VL) * BassGain + HPLeft.Filter(VL) * TrebleGain;
      VL:= VL * Preamp;

      if Clipped then
      begin
        if Abs(VL) > High(SmallInt) then
        begin
          VGL:= High(SmallInt) / Abs(VL);
          PeakGainLeft:= Min(PeakGainLeft, VGL);
        end;
        VL:= VL * PeakGainLeft;
      end else
      begin
        VL:=  Min(High(SmallInt), Max(VL, Low(SmallInt)));
      end;

      PV^:= Round(VL); Inc(PV);

      VR:= FFTRight.RealOut[I];
      VR:= VR + LPRight.Filter(VR) * BassGain + HPRight.Filter(VR) * TrebleGain;
      VR:= VR * Preamp;
      if Clipped then
      begin
        if Abs(VR) > High(SmallInt) then
        begin
          VGR:= High(SmallInt) / Abs(VR);
          PeakGainRight:= Min(PeakGainRight, VGR);
        end;
        VR:= VR * PeakGainRight;
      end else
      begin
        VR:= Min(High(SmallInt), Max(VR, Low(SmallInt)));
      end;

      PV^:= Round(VR); Inc(PV);
    end;

    PV:= PSmallInt(@Buffer);
    for I:= 0 to Min(FilterDataSize, FFTSpectrum1.BufferSize) - 1 do
    begin
      FFTSpectrum1.RealIn[I]:= PV^; Inc(PV);
      FFTSpectrum2.RealIn[I]:= PV^; Inc(PV);
    end;

    CalculateSpectrum(FFTSpectrum2);
    CalculateSpectrum(FFTSpectrum1);
  end;

begin
  //Exit;
  try
    FFTLeft.Clear; FFTRight.Clear;
    FFTSpectrum1.Clear; FFTSpectrum2.Clear;
    if NumOfChannels = 1 then ProcessMono else ProcessStereo;
  except
    StopButtonClick(nil);
    raise;
  end;
end;


procedure TMainForm.SetupProcessingParameters;
var P: Integer;
begin
  if SamplesPerSec > 0 then
  begin
    LPLeft.SampleRate:= SamplesPerSec;
    LPRight.SampleRate:= SamplesPerSec;
    HPLeft.SampleRate:= SamplesPerSec;
    HPRight.SampleRate:= SamplesPerSec;

    case SamplesPerSec of
      11025: P:= 8;
      22050: P:= 9;
      44100: P:= 10;
    else P:= 10;
    end;

    FFTSpectrum1.Power:= P - 1;
    FFTSpectrum2.Power:= FFTSpectrum1.Power;

    FFTEq.Power:= P;
    FFTLeft.Power:= P;
    FFTRight.Power:= P;
    FilterLen:= Round(Power(2, P - 2));
    GetMem(OverlapL, FilterLen * Sizeof(Single));
    GetMem(OverlapR, FilterLen * Sizeof(Single));
    FilterDataSize:= FFTLeft.BufferSize - FilterLen + 1;

    Audio.dspBufferSize:= FilterDataSize * 2 * NumOfChannels;

    for P:= 0 to FilterLen - 1 do
    begin
      OverlapL[P]:= 0;
      OverlapR[P]:= 0;
    end;

    TrackBar1Change(TB1);
  end;
end;


procedure TMainForm.Timer1Timer(Sender: TObject);
var
  I, Position: Integer;
  K: Single;
  IsFound: Boolean;
  SpectrumItem: PSpectrumItem;
  DispValues: PSpectrumArray;
begin
  IsFound:= false;
  CS.Acquire;
  try
    if SpectrumValues.Count > 0 then
    begin
      Position:= Audio.Position;
      while SpectrumValues.Count > 0 do
      begin
        SpectrumItem:= SpectrumValues[0];
        if SpectrumItem.Position <= Position then
        begin
          if SpectrumItem.Channel = 0 then DispValues:= @LeftDisp.Values
          else DispValues:= @RightDisp.Values;

          for I:= 0 to FFTSpectrum1.Power - 1 do
          begin
            if DispValues[I] < SpectrumItem.Values[I] then K:= 0.5 else K:= 0.1;
            DispValues[I]:= DispValues[I] + (SpectrumItem.Values[I] - DispValues[I])* K;
          end;

          SpectrumValues.Delete(0);
          IsFound:= true;
        end else
        begin
          Break;
        end;
      end;
    end else
    if (Audio = nil) or (Audio.Playing = false) then
    begin
      // HideValues
      for I:= 0 to FFTSpectrum1.Power - 1 do
      begin
        LeftDisp.Values[I]:= LeftDisp.Values[I] * 0.9;
        RightDisp.Values[I]:= RightDisp.Values[I] * 0.9;
        IsFound:= IsFound or
          (LeftDisp.Values[I] > 0.0001) or (RightDisp.Values[I] > 0.0001);
      end;
    end;
  finally
    CS.Release;
  end;

  if IsFound then
  begin
    LeftDisp.Paint;
    RightDisp.Paint;
  end;
end;


procedure TMainForm.TrackBar1Change(Sender: TObject);
var
  TB: TTrackBar;
  V: Single;

  procedure BuildFFTFilter;
  var I, Index, M, N, N1, N2: Integer;
  begin
    FFTEq.Clear;

    N:= 0; N1:= 1; N2:= 2;
    FFTEq.RealIn[N]:= 1;
    while N1 < FFTEq.BufferSize2 do
    begin
      for I:= N1 to N2 do FFTEq.RealIn[I]:= AmpCoeffs[N];

      Inc(N);
      N1:= N2;
      N2:= N2 * 2;
    end;

    for I:= 1 to FFTEq.BufferSize2 do
    begin
      FFTEq.RealIn[FFTEq.BufferSize - I]:=  FFTEq.RealIn[I];
      FFTEq.ImagIn[FFTEq.BufferSize - I]:= -FFTEq.ImagIn[I];
    end;

    FFTEq.IFFT;

    M:= FilterLen;
    for I:= 0 TO FFTEq.BufferSize - 1  do
    begin
     INDEX:= I + (M div 2);
     IF INDEX >= FFTEq.BufferSize THEN INDEX:= INDEX - FFTEq.BufferSize;
     FFTEq.ImagOut[INDEX]:= FFTEq.RealOut[I];
    end;

   FOR I:= 0 TO FFTEq.BufferSize - 1 do
   begin
     IF I <= M THEN FFTEq.RealIn[I]:= FFTEq.ImagOut[I]
     else FFTEq.RealIn[I]:= 0;
     FFTEq.ImagIn[I]:= 0;
   end;

   dspApplyWindow(fwHamming, PdspSingle(FFTEq.RealIn), M + 1);

   FFTEq.FFT;
 end;

begin
  TB:= TTrackBar(Sender);
  V:= Power(10, -TB.Position / 20);
  AmpCoeffs[TB.Tag]:= V;

  BuildFFTFIlter;

  TB.Hint:= Format('%d db', [-TB.Position]);
  {$IFNDEF VER120}
  Application.ActivateHint(Mouse.CursorPos);
  {$ENDIF}
end;


procedure TMainForm.TimbreChange(Sender: TObject);
begin
  BassGain:= Power(10, TBBass.Position / 20) - 1;
  TrebleGain:= Power(10, TBTreble.Position / 20) - 1;
  Preamp:= Power(10, TBPreamp.Position / 20);

  TBBass.Hint:= IntToStr(TBBass.Position) + ' dB';
  TBTreble.Hint:= IntToStr(TBTreble.Position) + ' dB';
  TBPreamp.Hint:= IntToStr(TBPreamp.Position) + ' dB';

  {$IFNDEF VER120}
  Application.ActivateHint(Mouse.CursorPos);
  {$ENDIF}
end;

procedure TMainForm.cbClipClick(Sender: TObject);
begin
  Clipped:= cbClip.Checked;
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
  fmAbout.ShowModal;
end;

end.
